@@ -17,6 +17,10 @@ BLUE = "#1f77b4"
 GREEN = "#2ca02c"
 BACKGROUND = "#FFFFFF"
 
+#static variables 
+COLUMNS_ACC = ["Name", "Description", "Quantity","Grouping", "Position_1", "Position_2", "Position_3", "Orientation_1", "Orientation_2", "Orientation_3"]
+COLUMNS_CHN = ["Name", "Description","Quantity","Grouping","Position_1", "Position_2", "Position_3", "Direction_1", "Direction_2","Direction_3"]
+
 class view3D():
     """
     A 3D display where structure, impacts, accelerometer and channels can be quickly displayed. Additionaly, all objects
@@ -122,14 +126,14 @@ class view3D():
         """
         Animate a mode shape in the 3D display.
         """
-        #TODO: Check why ann secondary returns error.
+
         frameperiod = 1.0 / self.modeshape_animation["fps"]
 
         now = time()
         nextframe = now + frameperiod
 
         ann = self.modeshape_animation["animation_pts"]
-        #ann_secondary = self.modeshape_animation["animation_pts_secondary"]
+        ann_secondary = self.modeshape_animation["animation_pts_secondary"]
 
         if self.take_gif:
             self.plot.open_gif(self.gif_dir)
@@ -137,21 +141,19 @@ class view3D():
         if self.modeshape_animation["scalars"]:
             set_lim = np.sqrt(np.mean(ann ** 2, axis=0))
             self.plot.update_scalar_bar_range(clim=[np.min(set_lim), np.max(set_lim)])
-            #if self.modeshape_animation["animate_secondary_mode_shape"]==True:
-            #    self.plot.update_scalar_bar_range(clim=[np.min(ann_secondary), np.max(ann_secondary)])
-
-
+            if self.modeshape_animation["animate_secondary_mode_shape"]==True:
+                self.plot.update_scalar_bar_range(clim=[np.min(ann_secondary), np.max(ann_secondary)])
 
         for i in range(ann.shape[2]):
             add_val = ann[:, :, i]
-            #add_val_secondary = ann_secondary[:, i]
+            add_val_secondary = ann_secondary[:, i]
 
             self.plot.update_coordinates(self.modeshape_animation["or_pts"] + add_val, mesh=self.modeshape_animation["mesh"],render = False)
             if self.modeshape_animation["scalars"]:
-                #if self.modeshape_animation["animate_secondary_mode_shape"]==True:
-                #    self.plot.update_scalars(add_val_secondary, mesh=self.modeshape_animation["mesh"] ,render = False) # for color changing
-                #else:
-                self.plot.update_scalars(np.sqrt(np.mean(add_val ** 2, axis=1)).reshape(self.modeshape_animation["or_pts"].shape[0]), mesh=self.modeshape_animation["mesh"] ,render = False) # for color changing
+                if self.modeshape_animation["animate_secondary_mode_shape"]==True:
+                    self.plot.update_scalars(add_val_secondary, mesh=self.modeshape_animation["mesh"] ,render = False) # for color changing
+                else: 
+                    self.plot.update_scalars(np.sqrt(np.mean(add_val ** 2, axis=1)).reshape(self.modeshape_animation["or_pts"].shape[0]), mesh=self.modeshape_animation["mesh"] ,render = False) # for color changing
 
             self.plot.render()
             if self.take_gif:
@@ -171,9 +173,9 @@ class view3D():
         Clear mode shape from the 3D display.
         """
 
-        self.plot.update_coordinates(self.modeshape_animation["or_pts"], mesh=self.modeshape_animation["mesh"],render = True)
-        self.plot.update_scalars(np.zeros(self.modeshape_animation["or_pts"].shape[0]), mesh=self.modeshape_animation["mesh"] ,render = False)
-        self.plot.update_scalar_bar_range(clim=[0,100])
+        self.plot.update_coordinates(self.modeshape_animation["or_pts"], mesh=self.modeshape_animation["mesh"], render=True)
+        self.plot.update_scalars(np.zeros(self.modeshape_animation["or_pts"].shape[0]), mesh=self.modeshape_animation["mesh"], render=False)
+        self.plot.update_scalar_bar_range(clim=[-100,100])
 
 
     def add_objects_animation(self,dict_animation,run_animation = False,add_note = False):
@@ -422,7 +424,7 @@ class view3D():
         i = int(len(self.all_accs_dynamic)+len(self.all_imps_dynamic)+len(self.all_vps_dynamic))
         size = 10
 
-        if orientation == None:
+        if np.asarray(orientation).all() == None:
             acc = self.create_accelerometer([size/2, size/2, size/2], [0, 0, 0], size=size)
             rot = np.diag([1]*3)
         else:
@@ -478,7 +480,7 @@ class view3D():
 
         i = int(len(self.all_accs_dynamic)+len(self.all_imps_dynamic)+len(self.all_vps_dynamic))
         size = 10
-        if direction == None:
+        if np.asarray(direction).all() == None:
             imp, _ = self.add_impact([size/2, size/2, size/2], [0, 0, 1], size=10)
             rot = np.diag([1]*3)
         else:
